#
# smartlog >>>version<<<
# (c) 1994 by Michael Weber
#
# original smartlog.awk written by Wolfgang Stumvoll
# (modified and extended)
#

#{{{}}}
#{{{  startup
BEGIN { 
   SCRIPT=">>>sm_path<<</smartlog.sh"
   FS=":"
   ORIGIN=""
#   {{{  set emergency defaults
   hold_def=hold_wrk="14"
   local_def=local_wrk="local=no"
   shift_def=shift_wrk="shift=no"
   comp_def=comp_wrk="comp=no"
   arch_def=arch_wrk="arch='none'"
   ash_def=ash_wrk="ash=none"
   bsh_def=bsh_wrk="bsh=none"
   nsh_def=nsh_wrk="nsh=none"
   zip_def=zip_wrk="zip=/bin/compress"
   zap_def=zap_wrk="zap=/bin/uncompress"
   ext_def=ext_wrk="ext=auto"
   dir_def=dir_wrk="dir='/var/log'"
   tmp_def=tmp_wrk="tmp='/tmp'"
   date_def=date_wrk="date=`/bin/date`"
   size_def=size_wrk="0"
#   }}}
   if (verbose>0)
      SCRIPT=SCRIPT" -v"
}
#}}}
#{{{  verbose comment
verbose==2 {
   if (file!=FILENAME) {
      file=FILENAME
      print("handling file "file) >>LOG
   }
   print("handling command '"$0"'") >>LOG
}
#}}}
#{{{  prepare lines for parsing
/^[ 	]*(#.*)?$/ { next }                          # skip comment/blank lines

#
# get a copy for working and cut trailing blanks
# using sub("[ 	]*$","") would be nice but it does not work
# because this stupid thing cuts off the stringterminator as well :-(
# so we have to use the WRK-crutch...
#
{ WRK=$0
  l=length(WRK)
  while(match(substr(WRK,l,1),"[    ]")) {
    l=l-1
    WRK=susbtr(WRK,l)
  }
}

WRK~/^[ 	]*/ { sub("[ 	]*","",WRK) }            # cut leading blanks
WRK~/^.*[^\\]#.*/ { sub("[      ]*[^\\\\]#.*","",WRK) }  # cut inline comments
WRK~/^.*[\\]#.*/ { gsub("[\\\\]#","#",WRK) }             # handle quoted hashes
#}}}
#{{{  overwrite defaults
#{{{  change hold default
WRK~/^hold=(def(ault)?)?$/ { hold_wrk=hold_def;next }
WRK~/^hold=[0-9]+$/ { hold_wrk=WRK;next }
WRK~/^[0-9]+$/ { hold_wrk="hold="WRK;next }
#}}}
#{{{  change local default
WRK~/^local=(def(ault)?)?$/ { local_wrk=local_def;next }
WRK~/^local=[nN]/ { local_wrk="local=no";next }
WRK~/^local(=[yY].*)?$/ { local_wrk="local=yes";next }
#}}}
#{{{  change shift default
WRK~/^shift=(def(ault)?)?$/ { shift_wrk=shift_def;next }
WRK~/^shift=[nN]/ { shift_wrk="shift=no";next }
WRK~/^shift(=[yY].*)?$/ { shift_wrk="shift=yes";next }
#}}}
#{{{  change comp default
WRK~/^comp(ress)?=(def(ault)?)?$/ { comp_wrk=comp_def;next }
WRK~/^comp(ress)?=[0-9]+$/ { comp_wrk="comp="substr(WRK,1+match(WRK,"="));next }
WRK~/^comp(ress)?=[nN]/ { comp_wrk="comp=no";next }
WRK~/^comp(ress)?(=[yY].*)?$/ { comp_wrk="comp=yes";next }
#}}}
#{{{  change arch default
WRK~/^arch(ive)?=(def(ault)?)?$/ { arch_wrk=arch_def;next }
WRK~/^arch(ive)?=[^'"`* 	]+$/ { arch_wrk="arch='"substr(WRK,1+match(WRK,"="))"'";next }
#}}}
#{{{  change ash default
WRK~/^ash=(def(ault)?)?$/ { ash_wrk=ash_def;next }
WRK~/^ash=/ {
   v=substr(WRK,5)
#   {{{  quote text
   for (of=1;;of=of+l+9) {
      l=match(substr(v,of),"'")
      if (l==0)
         break
      v=substr(v,1,of+l-2)"'\\'\\\\\\'\\''"substr(v,of+l)
   }
#   }}}
   ash_wrk="ash='"v"'"
   next
}
#}}}
#{{{  change bsh default
WRK~/^bsh=(def(ault)?)?$/ { bsh_wrk=bsh_def;next }
WRK~/^bsh=/ {
   v=substr(WRK,5)
#   {{{  quote text
   for (of=1;;of=of+l+9) {
      l=match(substr(v,of),"'")
      if (l==0)
         break
      v=substr(v,1,of+l-2)"'\\'\\\\\\'\\''"substr(v,of+l)
   }
#   }}}
   bsh_wrk="bsh='"v"'"
   next
}
#}}}
#{{{  change nsh default
WRK~/^nsh=(def(ault)?)?$/ { nsh_wrk=nsh_def;next }
WRK~/^nsh=/ {
   v=substr(WRK,5)
#   {{{  quote text
   for (of=1;;of=of+l+9) {
      l=match(substr(v,of),"'")
      if (l==0)
         break
      v=substr(v,1,of+l-2)"'\\'\\\\\\'\\''"substr(v,of+l)
   }
#   }}}
   nsh_wrk="nsh='"v"'"
   next
}
#}}}
#{{{  change zip default
WRK~/^zip=(def(ault)?)?$/ { zip_wrk=zip_def;next }
WRK~/^zip=[^ 	'"`]+$/ { zip_wrk="zip='"substr(WRK,5)"'";next }
#}}}
#{{{  change zap default
WRK~/^zap=(def(ault)?)?$/ { zap_wrk=zap_def;next }
WRK~/^zap=[^ 	'"`]+$/ { zap_wrk="zap='"substr(WRK,5)"'";next }
#}}}
#{{{  change ext default
WRK~/^ext=(def(ault)?)?$/ { ext_wrk=ext_def;next }
WRK~/^ext=[^ 	'"`]+$/ { ext_wrk="ext='"substr(WRK,5)"'";next }
#}}}
#{{{  change dir default
WRK~/^dir=(def(ault)?)?$/ { dir_wrk=dir_def;next }
WRK~/^dir=[^ 	'"`]+$/ { dir_wrk="dir='"substr(WRK,5)"'";next }
#}}}
#{{{  change tmp default
WRK~/^tmp=(def(ault)?)?$/ { tmp_wrk=tmp_def;next }
WRK~/^tmp=[^ 	'"`]+$/ { tmp_wrk="tmp='"substr(WRK,5)"'";next }
#}}}
#{{{  change date default
WRK~/^date=(def(ault)?)?$/ { date_wrk=date_def;next }
WRK~/^date=[^ 	]+/ {
   v=substr(WRK,6)
#   {{{  quote text
   for (of=1;;of=of+l+9) {
      l=match(substr(v,of),"'")
      if (l==0)
         break
      v=substr(v,1,of+l-2)"'\\'\\\\\\'\\''"substr(v,of+l)
   }
#   }}}
   date_wrk="date='"v"'"
   next
}

#}}}
#{{{  change size default
WRK~/^size=(def(ault)?)?$/ { size_wrk=size_def;next }
WRK~/^size=[0-9]+$/ { size_wrk=WRK;next }
#}}}
#{{{  DEFAULTS
WRK~/^DEFAULTS/ {
   hold_def=hold_wrk
   local_def=local_wrk
   shift_def=shift_wrk
   comp_def=comp_wrk
   arch_def=arch_wrk
   ash_def=ash_wrk
   bsh_def=bsh_wrk
   nsh_def=nsh_wrk
   zip_def=zip_wrk
   zap_def=zap_wrk
   ext_def=ext_wrk
   dir_def=dir_wrk
   tmp_def=tmp_wrk
   date_def=date_wrk
   size_def=size_wrk
   next
}
#}}}
#}}}
#{{{  check and parse logfile-lines
WRK~/^logfile=[^:'"` 	]+[ 	]*(:[ 	]*(\
((local|shift|comp(ress)?)(=([ynYN][^'"` 	]*)?)?)|\
(((comp(ress)?|size|hold)=)?[0-9]+)|\
((z[ia]p|ext|dir|tmp|arch(ive)?)=([^'"` 	]*)?)|\
(([abn]sh|date)(=(.+)?))\
))*$/ {
#  {{{  parse line and assign arguments
#  {{{  default init
  hold=hold_wrk
  local=local_wrk
  shift=shift_wrk
  comp=comp_wrk
  arch=arch_wrk
  ash=ash_wrk
  bsh=bsh_wrk
  nsh=nsh_wrk
  zip=zip_wrk
  zap=zap_wrk
  ext=ext_wrk
  dir=dir_wrk
  tmp=tmp_wrk
  date=date_wrk
  size=size_wrk
#  }}}
  for (i=1;i<=NF;i++) {
     v=$i
#     {{{  cut leading/trailing blanks
     sub("^[ 	]*","",v)

     l=length(v)
     while(match(substr(v,l,1),"[ 	]")) {
       l=l-1
       WRK=susbtr(v,l)
     }
#     }}}
#     {{{  quote text
     for (of=1;;of=of+l+9) {
        l=match(substr(v,of),"'")
        if (l==0)
           break
        v=substr(v,1,of+l-2)"'\\'\\\\\\'\\''"substr(v,of+l)
     }
#     }}}
     if (i==1)
        logfile="logfile="DQ substr(v,9) DQ
     else {
#        {{{  set hold arg
        if (match(v,"^hold=(def(ault)?)?$"))
           hold=hold_def
        else if (match(v,"^hold=[0-9]+$"))
           hold=v
        else if (match(v,"^[0-9]+$"))
           hold="hold="v
#        }}}
#        {{{  set local arg
        if (match(v,"^local=(def(ault)?)?$"))
           local=lcoal_def
        else if (match(v,"^local=[nN]"))
           local=local"local=no"
        else if (match(v,"^local(=[yY].*)?$"))
           local="local=yes"
#        }}}
#        {{{  set shift arg
        if (match(v,"^shift=(def(ault)?)?$"))
           shift=shift_def
        else if (match(v,"^shift=[nN]"))
           shift="shift=no"
        else if (match(v,"^shift(=[yY].*)?$"))
           shift="shift=yes"
#        }}}
#        {{{  set comp arg
        if (match(v,"^comp(ress)?=(def(ault)?)?$"))
           comp=comp_def
        else if (match(v,"^comp(ress)?=[nN]"))
           comp="comp=no"
        else if (match(v,"^comp(ress)?(=[yY].*)?$"))
           comp="comp=yes"
        else if (match(v,"^comp(ress)?=[0-9]+$"))
           comp="comp="substr(v,1+match(v,"="))
#        }}}
#        {{{  set arch arg
        if (match(v,"^arch(ive)?=(def(ault)?)?$"))
           arch=arch_def
        else if (match(v,"^arch(ive)?=\\*"))
           arch=arch DQ substr(v,2+match(v,"=")) DQ
        else if (match(v,"^arch(ive)?="))
           arch="arch="DQ substr(v,1+match(v,"=")) DQ
#        }}}
#        {{{  set shell args
        if (match(v,"^ash=(def(ault)?)?$"))
           ash=ash_def
        else if (match(v,"^ash="))
           ash="ash=\\''"substr(v,5)"'\\'"

        if (match(v,"^bsh=(def(ault)?)?$"))
           bsh=bsh_def
        else if (match(v,"^bsh="))
           bsh="bsh=\\''"substr(v,5)"'\\'"

        if (match(v,"^nsh=(def(ault)?)?$"))
           nsh=nsh_def
        else if (match(v,"^nsh="))
           nsh="nsh=\\''"substr(v,5)"'\\'"
#        }}}
#        {{{  set zip arg
        if (match(v,"^zip=(def(ault)?)?$"))
           zip=zip_def
        else if (match(v,"^zip="))
           zip="zip='"substr(v,5)"'"
#        }}}
#        {{{  set zap arg
        if (match(v,"^zap=(def(ault)?)?$"))
           zap=zap_def
        else if (match(v,"^zap="))
           zap="zap='"substr(v,5)"'"
#        }}}
#        {{{  set ext arg
        if (match(v,"^ext=(def(ault)?)?$"))
           ext=ext_def
        else if (match(v,"^ext="))
           ext="ext='"substr(v,5)"'"
#        }}}
#        {{{  set dir arg
        if (match(v,"^dir=(def(ault)?)?$"))
           dir=dir_def
        else if (match(v,"^dir="))
           dir="dir=\\''"substr(v,5)"'\\'"
#        }}}
#        {{{  set tmp arg
        if (match(v,"^tmp=(def(ault)?)?$"))
           tmp=tmp_def
        else if (match(v,"^tmp="))
           tmp="tmp=\\''"substr(v,5)"'\\'"
#        }}}
#        {{{  set date arg
        if (match(v,"^date=(def(ault)?)?$"))
           date=date_def
        else if (match(v,"^date="))
           date="date='"substr(v,6)"'"
#        }}}
#        {{{  set size arg
        if (match(v,"^size=(def(ault)?)?$"))
           size=size_def
        else if (match(v,"^size=[0-9]+$"))
           size=v
#        }}}
     }
  }
#  }}}
#  {{{  create commandline and call smartlog.sh
  cmd=SCRIPT" "DQ logfile DQ" "DQ hold DQ" "DQ local DQ" "DQ shift DQ" "DQ comp DQ" "DQ arch DQ" "DQ bsh DQ" "DQ nsh DQ" "DQ ash DQ" "DQ zip DQ" "DQ zap DQ" "DQ ext DQ" "DQ dir DQ" "DQ tmp DQ" "DQ date DQ" "DQ size DQ
  if (verbose==2)
     print("calling '"cmd"'") >>LOG

  close(LOG)

  if (noexec==1)
     e=0
  else
     e=system(cmd)

  if (e!=0) {
     print("exit code "e) >>LOG
     if (verbose==0)
        print("error while executing: "cmd) >>LOG
#  }}}
  }
  next
}
#}}}
#{{{  complain about invalid lines
{
   print("Invalid line in "FILENAME", line "FNR": "$0) >>LOG
   next
}
#}}}
