#! /bin/sh
: &&O='cd .' || exec /bin/sh "$0" $argv:q # we're in a csh, feed myself to sh
$O || exec /bin/sh "$0" "$@"              # we're in a buggy zsh

#
# smartlog >>>version<<<
# (c) 1994 by Michael Weber
#

#{{{}}}
#{{{  init
#{{{  parameters
#{{{  check for -v
if [ x"$1" = x"-v" ]
then
  verbose="y"
  shift
else
  verbose=""
fi
#}}}
eval "$@"
#}}}
#{{{  variables
sm_path=">>>sm_path<<<"
sm_user="$HOME/.smartlog"
sm_logfile="$sm_user/logfile"
sm_get="$sm_path/getmode"

LS=">>>ls<<<"
RM=">>>rm<<< -f"
MV=">>>mv<<< -f"
CAT=">>>cat<<< --"
SED=">>>sed<<<"
AWK=">>>awk<<<"
EXPR=">>>expr<<<"
GREP=">>>grep<<<"
CHOWN=">>>chown<<<"
CHGRP=">>>chgrp<<<"
CHMOD=">>>chmod<<<"
DIRNAME=">>>dirname<<<"

[ "$local" = "no" ] && local=""
[ "$shift" = "no" ] && shift=""
[ "$comp" = "no" ] && comp=""
[ "$ash" = "none" ] && ash=""
[ "$bsh" = "none" ] && bsh=""
[ "$nsh" = "none" ] && nsh=""
[ "$arch" = "none" ] && arch=""

log=""
magiclog=""
suffix=".SLA"
suffix2="$suffix"   # needed for comparisons
identifier="##### SLA #####"

trap '$RM "$tmp"; panic "TRAP-EXIT"' 1 2 3 15
#}}}

#{{{  assign()
assign()
{ fuid="$1"
  fgid="$2"
  fsize="$3"
  fmode="$4"
}
#}}}
#{{{  calc()
calc()
{ if [ -z "$EXPR" ]
  then
    result=$[$*]
  else
    result=`$EXPR $*`
  fi
}
#}}}
#{{{  compress()
compress()
{ if [ "$comp" -a ! -z "$ext" ]
  then
    $zip <"$1" >"$1$ext"
    $RM "$1"
    restore "$1$ext"
  fi
}
#}}}
#{{{  ftouch()
#
# emulating "touch" using "cat"
# why to use more systools than absolutely necessary?
#
ftouch()
{ $CAT </dev/null >"$1"
}
#}}}
#{{{  move()
#
# shiftlog function
#
move()
{ if [ -f "$1" ]
  then
    $MV "$1" "$2"
    compress "$2"
  elif [ -f "$1$ext" ]
    then
      $MV "$1$ext" "$2$ext"
      restore "$2$ext"
  fi
}
#}}}
#{{{  panic()
panic()
{ echo "$date: Panic: $*." >>$sm_logfile
  $RM "$tmp" # clear tmpfile
  exit 1
}
#}}}
#{{{  restore()
restore()
{ [ -f "$1" ] || ftouch "$1"
  $CHOWN "$fuid" "$1"
  $CHGRP "$fgid" "$1"
  $CHMOD "$fmode" "$1"
}
#}}}
#{{{  uncompress()
uncompress()
{ if [ -r "$1$ext2" ]
  then
    $zap <"$1$ext2" >"$1"
    $RM "$1$ext2"
    restore "$1"
  fi
}
#}}}
#{{{  warn()
warn()
{ echo "$date: Warning: $*." >> $sm_logfile
}
#}}}

#{{{  check parameter integrity
#{{{  check/set tmp
if [ -d "$tmp" ]
then
  [ -w "$tmp" ] || panic "$tmp is not writable"
  tmp="$tmp/smartlog.$$"
fi
$RM "$tmp" # just to be sure we have no tmp-file lazing around
#}}}
#{{{  check logfile and archdir
[ -f "$logfile" ] || panic "$logfile is not a regular file or does not exist"
[ -r "$logfile" ] || panic "$logfile is not readable"
[ -f "$dir" ] && panic "archive dir $dir is a regular file"
[ -w "$dir" ] || panic "archive dir $dir does not exist or is unwritable"
#}}}
#{{{  check zip/zap
[ "x`(echo 'foo' | $zip | $zap) 2>/dev/null`" != "xfoo" ] && panic "zip/zap do not exist or are incompatible"
#}}}
#{{{  set ext if necessary
#
# "none" is for compatibility to prior versions (2.1 or below)
# "auto" sounds more logical, ain't it?!
#
if [ x"$ext" = x"auto" -o x"$ext" = x"none" ]
then
#  {{{  compress a file and look for the suffix
#  
#   a file called "99999" should not collide with any other
#   (hopefully :-)
#  
  echo "--------------------------------------------------------\
        Some programs do not compress empty or very small files.\
        But this size should be ok, even for a dumb 'compress'.
        --------------------------------------------------------" > "99999"
  $zip "99999" 2>&1 >/dev/null
  ext=`$LS 99999* | $SED "s/9//g"`
  $RM 99999*
#  }}}
#  {{{  if $zip gave no hint, we try something else
  if [ -z "$ext" ]
  then
    case $zip in
      *compress) ext=".Z";;
      *gzip) ext=".gz";;
      *zip) ext=".zip";;
      *freeze) ext=".F";;
      *compact) ext=".C";;  # just for nostalgia :-)
      *) ext=".SLAZ";warn "cannot determine suffix for $zip-$zap, using .SLAZ";;
    esac
  fi
#  }}}
fi
ext2="$ext" # needed for comparisons
#}}}
#}}}
#{{{  get the files' mode and size
#
# assign will set fuid, fgid and fmode to the
# files' permissions which will be used to restore
# a logfiles' original permissions and ownership
# fsize wil be set to the files' size
#
assign `$sm_get $logfile`
#}}}
#}}}

if [ "$fsize" -ge "$size" ]
then
#  {{{  preparatory work
#  {{{  what is the archives' name?
#  {{{  are we in shifting-mode?
  [ "$shift" ] && suffix=""   # yep, no suffix needed
#  }}}
  if [ -z "$arch" ]
  then
#    {{{  no alternative archive given
    if [ "$local" ]
    then
      archive="$logfile"
    else
      archive="$dir/"`echo "$logfile" | $SED -e "s/\//./g" -e "/./s/.//"`
    fi
    archive2="$archive" # save archive without suffix
    archive="$archive$suffix"
#    }}}
  else
#    {{{  alternative archive given
    if [ "$local" ]
    then
      archive=`$DIRNAME "$logfile"`"/$arch"
    else
      archive="$dir/$arch"
    fi
    archive2="$archive" # save archive without suffix
#    }}}
  fi
#  }}}
#  {{{  checkups
#  {{{  archivepath
  [ ! -w `$DIRNAME $archive` ] && panic "invalid archive-path ($archive)"
#  }}}
#  {{{  logfile==archive
  [ x"$archive" = x"$logfile" -a ! "$shift" ] && panic "archive and logfile are identical for $logfile"
#  }}}
#  {{{  double archives
#  
#   these checks are *very* rough
#   but should suffice
#  
  [ -f "$archive" -a -f "$archive2$suffix2" -a \
    x"$archive" != x"$archive2$suffix2" -a \
    x"$archive" != x"$logfile" \
  ] && panic "double archive $archive & $archive2$suffix2"

  [ -f "$archive" -a -f "$archive2" -a \
    x"$archive" != x"$archive2" -a \
    x"$archive2" != x"$logfile" \
  ] && panic "double archive $archive & $archive2"
#  }}}
#  }}}
#  {{{  a present for archiving junkies
#  
#   if someone archives smartlog-archives (NO COMMENT!!!)
#   we have to adapt the identifier and the delimiter
#   this will even work for the archive of an archive of an archive ...
#  
  while [ `$GREP -c "$identifier" "$logfile"` -gt 0 ]
  do
    identifier="#####$identifier"    # grow identifier significantly
  done

  delimiter="$identifier=============== $date ==============="
#  }}}
#  }}}
#  {{{  archiving routines
#  {{{  bsh
#  
#   I know that two subshells are not going to win a cutie-prize
#   but they make sure all parts will be processed correctly
#  
  [ ! -z "$bsh" ] && (($bsh 2>>"$sm_logfile") || warn "bsh: '$bsh' exited with status $?")
#  }}}
  if [ "$shift" ]
  then
#    {{{  shifting mode
#    
#     original shiftlog routines by Stephen R. van den Berg
#     (modified)
#    
#    {{{  convert standard-archive if any
#    {{{  determine archive to convert
    if [ "$arch" ]
    then
      convarch="$archive"
    else
      convarch="$archive$suffix2"
    fi
    uncompress "$convarch"
#    }}}
    if [ -f "$convarch" ]
    then
#      {{{  panic on existing shift-archives
#      
#       this will kick you out if you somehow made it to create
#       standard and shift archives with the same name
#      
      [ -f "$archive2.0" ] && panic "convert: $archive2.* already exist"
#      }}}
#      {{{  awk does it
      $AWK -v IDENT="$identifier" -v DEST="$archive2"\
        'BEGIN { i=-1; ok=0 }\
        index($0,IDENT)==1 { close(DEST"."i); i=i+1; ok=1; next }\
        ok==1 { print >>DEST"."i }' <"$convarch"
#      }}}
      $RM "$convarch"
#      {{{  give all archives the right mode
      i=0
      while [ -f $archive.$i ]
      do
        restore "$archive.$i"
        calc $i + 1; i=$result
      done
#      }}}
      magiclog=$magiclog"/$convarch split into $i files"
    fi
#    }}}
#    {{{  compress || !compress
    if [ "$comp" ]
    then
      case "$comp" in
        *[!0-9]*) firstcomp=0;;  # no integer, compress all archives
        *) firstcomp=$comp;; # interpret integer variables
      esac
    else
      ext=""
      firstcomp=0 # make sure integer comparisons won't fail
    fi
#    }}}
#    {{{  cycle old archives
    i="$hold"  # start with the HIGHEST value (=LAST archive)
    while [ $i -gt 0 ]
    do
      calc $i - 1; ip=$result
#      {{{  uncompress archive if necessary
#      
#       archive with minor numbers maybe compressed
#       and we are uncompressing then now
#      
      [ -z "$ext" ] && uncompress "$archive.$ip"
#      }}}
      move "$archive.$ip" "$archive.$i"
      [ $i = $firstcomp ] && ext=""  # no compression on minor numbers
      i=$ip
    done
#    }}}
#    {{{  move the logfile to the archive
    move "$logfile" "$archive.0"
    restore "$logfile"    # recreate $logfile with its exact permissions
#    }}}
#    {{{  kill overhanging shift-archives
    calc $hold + 1; i=$result # we look for archives >max hold

    while [ -f "$archive.$i" -o -f "$archive.$i$ext2" ]
    do
#      
#       i know its not nice to remove both if one was detected,
#       but it makes the stuff more readable
#      
      $RM "$archive.$i" "$archive.$i$ext2"
      calc $i + 1; i=$result
    done
#    }}}
#    }}}
  else
#    {{{  standard mode
#    {{{  uncompress archive if any
    uncompress "$archive"
    uncompress "$archive2$suffix"
#    }}}
#    {{{  panic on existing standard AND shift archive
#    
#     this will kick you out if you somehow made it to create
#     standard and shift archives with the same name
#    
    [ -f "$archive" -a -f "$archive2.0" ] && \
      panic "convert: $archive already exists"
#    }}}
#    {{{  convert shift-archives if any
    i=0
    $RM "$tmp" # just to be sure
    while [ -f "$archive2.$i" -o -f "$archive2.$i$ext2" ]
    do
#      {{{  collect archives in tmp-file
      uncompress "$archive2.$i"
      echo "$delimiter" >> "$tmp"
      $CAT "$archive2.$i" >> "$tmp"
      $RM "$archive2.$i"
      calc $i + 1; i=$result
#      }}}
    done
#    {{{  save stuff if we have some
    if [ $i -gt 0 ]
    then
      $MV "$tmp" "$archive"
      magiclog=$magiclog"/$i files collected"
    fi
#    }}}
#    }}}
#    {{{  restore archives' permissions or touch it
    restore "$archive"
#    }}}
#    {{{  delete old entries
    held=`$GREP -c "$identifier" "$archive" 2>/dev/null`
    calc $held - $hold + 1

    if [ $result -gt 0 ]
    then
#      {{{  awk call to cut the leading generations
#      
#       you wonder about the double DEL==0 line? makes is fast!
#       the first one prints out the rest of the file
#       the second one saves the delimiter of the first generation
#       staying in the archive
#      
      $AWK -v IDENT="$identifier" -v DEL="$result"\
       'BEGIN { DEL=DEL+1 }\
        DEL==0 { print; next }\
        index($0,IDENT)==1 { DEL=DEL-1 }\
        DEL==0 { print }' <"$archive" >"$tmp"
#      }}}
      $MV "$tmp" "$archive" # move modified file back to origin
    fi
#    }}}
#    {{{  collect archive and logfile in a new archive
    echo "$delimiter" >> "$archive"
    $CAT "$archive" "$logfile" > "$tmp"
    $CAT </dev/null >"$logfile"  # this saves the logfiles' permissions
    $MV "$tmp" "$archive"
    restore "$archive"   # we want archives with logfile-permissions
    compress "$archive"  # compress archive if necessary
#    }}}
#    }}}
  fi
#  {{{  ash
  [ ! -z "$ash" ] && (($ash 2>>"$sm_logfile") || warn "ash: '$ash' exited with status $?")
#  }}}
#  }}}
#  {{{  diagnostic output for verbose mode
  if [ "$verbose" ]
  then
#    {{{  rig up output
    log="$logfile ($fsize bytes) --> $archive (maxhold $hold)"
    [ "$local" ] && log=$log"/local"
    [ "$shift" ] && log=$log"/shift"
    if [ "$comp" ]
    then
     log=$log"/compress"
     [ "$shift" ] && log=$log" (first #$firstcomp)"
    fi
    [ "$bsh" ] && log=$log"/bsh='$bsh'"
    [ "$ash" ] && log=$log"/ash='$ash'"
    [ "$nsh" ] && log=$log"/(nsh='$nsh')"
    log="$log$magiclog" # append info from automagical functions
#    }}}
    echo "$date: $log" >>"$sm_logfile"
  fi
#  }}}
else
#  {{{  file too small, execute nsh?
  if [ "$size" = 1 ]
  then
    echo "$date: $logfile is empty." >>$sm_logfile
  else
    echo "$date: $logfile has size below $size bytes." >>$sm_logfile
  fi
  
  if [ "$nsh" ]
  then
    echo "$date: executing '$nsh' instead." >>$sm_logfile
    (($nsh 2>>"$sm_logfile") || warn "nsh: '$nsh' exited with status $?")
  fi
#  }}}
fi

$RM "$tmp" # just to be *really* sure

exit 0
